function [b, alphas] = smo_train(X, Y, C, tol, max_passes)

% Runs the SMO algorithm.  X is the matrix of training examples.  Each row is
% a training example, and the jth column holds the jth feature.  Y is a column
% matrix containing 1 for positive examples and -1 for negative examples.  C
% is the standard SVM regularization parameter.  tol is a tolerance value used
% for determining equality of floating point numbers. 


% YOUR CODE HERE
b = 0;
alphas = zeros(size(X,1),1);
passes = 0;
m = size(X,1);
while (passes < max_passes)
   num_chnaged_alphas = 0;
   for i=1:m
       f_xi = sum(alphas.*Y.*(X*X(i,:)')) + b;  %calc f(xi)
       Ei = f_xi - Y(i);                 %calc E(i)=f(xi)-yi
       if ((Y(i)*Ei < -tol) && (alphas(i) < C)) || ((Y(i)*Ei > tol) && (alphas(i) > 0)) 
           j = j_rand_not_i(i,m); %select j!=i randomly
           f_xj = sum(alphas.*Y.*(X*X(j,:)')) + b;  %calc f(xj)
           Ej = f_xj - Y(j);
           alpha_i_old = alphas(i);
           alpha_j_old = alphas(j);
           if Y(i) ~= Y(j)
               L = max(0, alphas(j) - alphas(i));
               H = min(C, C + alphas(j) - alphas(i));
           else
               L = max(0, alphas(j) + alphas(i) - C);
               H = min(C, alphas(j) + alphas(i));
           end
           if L == H
               break
           end
           eta = 2 * X(i,:)*X(j,:)' - X(i,:)*X(i,:)' - X(j,:)*X(j,:)';
           if eta > 0
               break
           end
           alphas(j) = alphas(j) - Y(j)*(Ei - Ej)/eta;
           alphas(j) = clip_alpha(alphas(j),H,L);
           if (abs(alphas(j) - alpha_j_old) < 1e-5)
               break
           end
           alphas(i) = alphas(i) + Y(i)*Y(j)*(alpha_j_old - alphas(j));
           b1 = b - Ei - Y(i)*(alphas(i) - alpha_i_old)*X(i,:)*X(i,:)'-...
               Y(j)*(alphas(j) - alpha_j_old)*X(i,:)*X(j,:)';
           b2 = b - Ej - Y(i)*(alphas(i) - alpha_i_old)*X(i,:)*X(j,:)'-...
               Y(j)*(alphas(j) - alpha_j_old)*X(j,:)*X(j,:)';
           if (0 < alphas(i)) && (alphas(i) < C)
               b = b1;
           elseif (0 < alphas(j)) && (alphas(j) < C)
               b = b2;
           else
               b = (b1 + b2)/2;
           end
           num_chnaged_alphas = num_chnaged_alphas + 1;
       end  % end if
   end  %end for
   if num_chnaged_alphas == 0
       passes = passes + 1
   else
       passes = 0;
   end
end  %end while